from pyiso20022.colr.colr_008_001_05.colr_008_001_05 import (
    AccountIdentification4Choice,
    AccountSchemeName1Choice,
    CashCollateralResponse2,
    CollateralAccount3,
    CollateralAccountIdentificationType3Choice,
    CollateralAccountType1Code,
    CollateralProposalResponse1Code,
    CollateralProposalResponse3,
    CollateralProposalResponse3Choice,
    CollateralProposalResponseType3,
    CollateralProposalResponseV05,
    CollateralResponse2,
    DateAndDateTimeChoice,
    Document,
    ExposureType5Code,
    GenericAccountIdentification1,
    GenericIdentification36,
    NameAndAddress6,
    Obligation5,
    OtherCollateralResponse2,
    PartyIdentification100Choice,
    PostalAddress2,
    RejectionReasonV021Code,
    SecuritiesCollateralResponse1,
    Status4Code,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "AccountIdentification4Choice",
    "AccountSchemeName1Choice",
    "CashCollateralResponse2",
    "CollateralAccount3",
    "CollateralAccountIdentificationType3Choice",
    "CollateralAccountType1Code",
    "CollateralProposalResponse1Code",
    "CollateralProposalResponse3",
    "CollateralProposalResponse3Choice",
    "CollateralProposalResponseType3",
    "CollateralProposalResponseV05",
    "CollateralResponse2",
    "DateAndDateTimeChoice",
    "Document",
    "ExposureType5Code",
    "GenericAccountIdentification1",
    "GenericIdentification36",
    "NameAndAddress6",
    "Obligation5",
    "OtherCollateralResponse2",
    "PartyIdentification100Choice",
    "PostalAddress2",
    "RejectionReasonV021Code",
    "SecuritiesCollateralResponse1",
    "Status4Code",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from pyiso20022.admi.admi_004_001_02.admi_004_001_02 import (
    Document,
    Event2,
    SystemEventNotificationV02,
)

__all__ = [
    "Document",
    "Event2",
    "SystemEventNotificationV02",
]



from pyiso20022.colr.colr_009_001_05.colr_009_001_05 import (
    ActiveCurrencyAndAmount,
    BlockChainAddressWallet5,
    CollateralAccount3,
    CollateralAccountIdentificationType3Choice,
    CollateralAccountType1Code,
    DateAndDateTime2Choice,
    Dispute1,
    DisputeNotification2,
    DisputeNotification2Choice,
    DisputeResolutionType1Choice,
    DisputeResolutionType1Code,
    DisputeResolutionType2Choice,
    DisputeResolutionType2Code,
    Document,
    ExposureType11Code,
    GenericIdentification30,
    GenericIdentification36,
    MarginCallDisputeNotificationV05,
    NameAndAddress6,
    Obligation9,
    PartyIdentification178Choice,
    PostalAddress2,
    SegregatedIndependentAmountDispute2,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    VariationMarginDispute1,
)

__all__ = [
    "ActiveCurrencyAndAmount",
    "BlockChainAddressWallet5",
    "CollateralAccount3",
    "CollateralAccountIdentificationType3Choice",
    "CollateralAccountType1Code",
    "DateAndDateTime2Choice",
    "Dispute1",
    "DisputeNotification2",
    "DisputeNotification2Choice",
    "DisputeResolutionType1Choice",
    "DisputeResolutionType1Code",
    "DisputeResolutionType2Choice",
    "DisputeResolutionType2Code",
    "Document",
    "ExposureType11Code",
    "GenericIdentification30",
    "GenericIdentification36",
    "MarginCallDisputeNotificationV05",
    "NameAndAddress6",
    "Obligation9",
    "PartyIdentification178Choice",
    "PostalAddress2",
    "SegregatedIndependentAmountDispute2",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "VariationMarginDispute1",
]



from pyiso20022.colr.colr_008_001_03.colr_008_001_03 import (
    AccountIdentification4Choice,
    AccountSchemeName1Choice,
    CashCollateralResponse1,
    CollateralAccount1,
    CollateralAccountIdentificationType1Choice,
    CollateralAccountType1Code,
    CollateralProposalResponse1Code,
    CollateralProposalResponse2,
    CollateralProposalResponse2Choice,
    CollateralProposalResponseType2,
    CollateralProposalResponseV03,
    CollateralResponse1,
    DateAndDateTimeChoice,
    Document,
    ExposureType5Code,
    GenericAccountIdentification1,
    GenericIdentification29,
    NameAndAddress6,
    Obligation3,
    OtherCollateralResponse1,
    PartyIdentification33Choice,
    PostalAddress2,
    RejectionReasonV021Code,
    SecuritiesCollateralResponse1,
    Status4Code,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "AccountIdentification4Choice",
    "AccountSchemeName1Choice",
    "CashCollateralResponse1",
    "CollateralAccount1",
    "CollateralAccountIdentificationType1Choice",
    "CollateralAccountType1Code",
    "CollateralProposalResponse1Code",
    "CollateralProposalResponse2",
    "CollateralProposalResponse2Choice",
    "CollateralProposalResponseType2",
    "CollateralProposalResponseV03",
    "CollateralResponse1",
    "DateAndDateTimeChoice",
    "Document",
    "ExposureType5Code",
    "GenericAccountIdentification1",
    "GenericIdentification29",
    "NameAndAddress6",
    "Obligation3",
    "OtherCollateralResponse1",
    "PartyIdentification33Choice",
    "PostalAddress2",
    "RejectionReasonV021Code",
    "SecuritiesCollateralResponse1",
    "Status4Code",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from pyiso20022.admi.admi_006_001_01.admi_006_001_01 import (
    AddressType2Code,
    Document,
    GenericIdentification1,
    GenericIdentification36,
    MessageHeader7,
    NameAndAddress5,
    OriginalBusinessQuery1,
    PartyIdentification120Choice,
    PartyIdentification136,
    PostalAddress1,
    RequestType4Choice,
    ResendRequestV01,
    ResendSearchCriteria2,
    SequenceRange1,
    SequenceRange1Choice,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "AddressType2Code",
    "Document",
    "GenericIdentification1",
    "GenericIdentification36",
    "MessageHeader7",
    "NameAndAddress5",
    "OriginalBusinessQuery1",
    "PartyIdentification120Choice",
    "PartyIdentification136",
    "PostalAddress1",
    "RequestType4Choice",
    "ResendRequestV01",
    "ResendSearchCriteria2",
    "SequenceRange1",
    "SequenceRange1Choice",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from __future__ import annotations

from dataclasses import dataclass, field
from decimal import Decimal
from enum import Enum

from xsdata.models.datatype import XmlDate, XmlDateTime

__NAMESPACE__ = "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04"


@dataclass(kw_only=True)
class ActiveCurrencyAndAmount:
    value: Decimal = field(
        metadata={
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        }
    )
    ccy: str = field(
        metadata={
            "name": "Ccy",
            "type": "Attribute",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


@dataclass(kw_only=True)
class ActiveOrHistoricCurrencyAndAmount:
    value: Decimal = field(
        metadata={
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        }
    )
    ccy: str = field(
        metadata={
            "name": "Ccy",
            "type": "Attribute",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


class AgreementFramework1Code(Enum):
    FBAA = "FBAA"
    BBAA = "BBAA"
    DERV = "DERV"
    ISDA = "ISDA"
    NONR = "NONR"


class CollateralAccountType1Code(Enum):
    HOUS = "HOUS"
    CLIE = "CLIE"
    LIPR = "LIPR"
    MGIN = "MGIN"
    DFLT = "DFLT"


class CreditDebitCode(Enum):
    CRDT = "CRDT"
    DBIT = "DBIT"


@dataclass(kw_only=True)
class DateAndDateTimeChoice:
    dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "Dt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "DtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class DatePeriodDetails:
    fr_dt: XmlDate = field(
        metadata={
            "name": "FrDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    to_dt: XmlDate = field(
        metadata={
            "name": "ToDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


class ExposureType5Code(Enum):
    BFWD = "BFWD"
    PAYM = "PAYM"
    CCPC = "CCPC"
    COMM = "COMM"
    CRDS = "CRDS"
    CRTL = "CRTL"
    CRSP = "CRSP"
    CCIR = "CCIR"
    CRPR = "CRPR"
    EQUI = "EQUI"
    EQPT = "EQPT"
    EQUS = "EQUS"
    EXTD = "EXTD"
    EXPT = "EXPT"
    FIXI = "FIXI"
    FORX = "FORX"
    FORW = "FORW"
    FUTR = "FUTR"
    OPTN = "OPTN"
    LIQU = "LIQU"
    OTCD = "OTCD"
    REPO = "REPO"
    RVPO = "RVPO"
    SLOA = "SLOA"
    SBSC = "SBSC"
    SCRP = "SCRP"
    SLEB = "SLEB"
    SHSL = "SHSL"
    SCIR = "SCIR"
    SCIE = "SCIE"
    SWPT = "SWPT"
    TBAS = "TBAS"
    TRBD = "TRBD"
    TRCP = "TRCP"


class Frequency1Code(Enum):
    YEAR = "YEAR"
    MNTH = "MNTH"
    QURT = "QURT"
    MIAN = "MIAN"
    WEEK = "WEEK"
    DAIL = "DAIL"
    ADHO = "ADHO"
    INDA = "INDA"


@dataclass(kw_only=True)
class GenericIdentification30:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "pattern": r"[a-zA-Z0-9]{4}",
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification36:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class Pagination:
    pg_nb: str = field(
        metadata={
            "name": "PgNb",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "pattern": r"[0-9]{1,5}",
        }
    )
    last_pg_ind: bool = field(
        metadata={
            "name": "LastPgInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PostalAddress2:
    strt_nm: None | str = field(
        default=None,
        metadata={
            "name": "StrtNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 70,
        },
    )
    pst_cd_id: str = field(
        metadata={
            "name": "PstCdId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 16,
        }
    )
    twn_nm: str = field(
        metadata={
            "name": "TwnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    ctry_sub_dvsn: None | str = field(
        default=None,
        metadata={
            "name": "CtrySubDvsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry: str = field(
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "pattern": r"[A-Z]{2,2}",
        }
    )


@dataclass(kw_only=True)
class SupplementaryDataEnvelope1:
    any_element: None | object = field(
        default=None,
        metadata={
            "type": "Wildcard",
            "namespace": "##any",
        },
    )


@dataclass(kw_only=True)
class AgreementFramework1Choice:
    agrmt_frmwk: None | AgreementFramework1Code = field(
        default=None,
        metadata={
            "name": "AgrmtFrmwk",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    prtry_id: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class AmountAndDirection20:
    amt: ActiveOrHistoricCurrencyAndAmount = field(
        metadata={
            "name": "Amt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    cdt_dbt_ind: None | CreditDebitCode = field(
        default=None,
        metadata={
            "name": "CdtDbtInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class CollateralAccountIdentificationType2Choice:
    tp: None | CollateralAccountType1Code = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    prtry: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class NameAndAddress6:
    nm: str = field(
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 70,
        }
    )
    adr: PostalAddress2 = field(
        metadata={
            "name": "Adr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class Statement32:
    stmt_id: str = field(
        metadata={
            "name": "StmtId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    actvty_ind: bool = field(
        metadata={
            "name": "ActvtyInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    frqcy: Frequency1Code = field(
        metadata={
            "name": "Frqcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    stmt_dt_tm: DateAndDateTimeChoice = field(
        metadata={
            "name": "StmtDtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class SupplementaryData1:
    plc_and_nm: None | str = field(
        default=None,
        metadata={
            "name": "PlcAndNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 350,
        },
    )
    envlp: SupplementaryDataEnvelope1 = field(
        metadata={
            "name": "Envlp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class Agreement4:
    agrmt_dtls: str = field(
        metadata={
            "name": "AgrmtDtls",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 140,
        }
    )
    agrmt_id: None | str = field(
        default=None,
        metadata={
            "name": "AgrmtId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 140,
        },
    )
    agrmt_dt: XmlDate = field(
        metadata={
            "name": "AgrmtDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    base_ccy: str = field(
        metadata={
            "name": "BaseCcy",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )
    agrmt_frmwk: None | AgreementFramework1Choice = field(
        default=None,
        metadata={
            "name": "AgrmtFrmwk",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class CollateralAccount2:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    tp: None | CollateralAccountIdentificationType2Choice = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class PartyIdentification100Choice:
    any_bic: None | str = field(
        default=None,
        metadata={
            "name": "AnyBIC",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "pattern": r"[A-Z]{6,6}[A-Z2-9][A-NP-Z0-9]([A-Z0-9]{3,3}){0,1}",
        },
    )
    prtry_id: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    nm_and_adr: None | NameAndAddress6 = field(
        default=None,
        metadata={
            "name": "NmAndAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class InterestCalculation4:
    clctn_dt: XmlDate = field(
        metadata={
            "name": "ClctnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    coll_acct_id: None | CollateralAccount2 = field(
        default=None,
        metadata={
            "name": "CollAcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    fctv_prncpl_amt: AmountAndDirection20 = field(
        metadata={
            "name": "FctvPrncplAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    prncpl_amt: None | AmountAndDirection20 = field(
        default=None,
        metadata={
            "name": "PrncplAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    mvmnt_amt: None | AmountAndDirection20 = field(
        default=None,
        metadata={
            "name": "MvmntAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    nb_of_days: None | Decimal = field(
        default=None,
        metadata={
            "name": "NbOfDays",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "total_digits": 18,
            "fraction_digits": 0,
        },
    )
    fctv_rate: Decimal = field(
        metadata={
            "name": "FctvRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "total_digits": 11,
            "fraction_digits": 10,
        }
    )
    intrst_rate: None | Decimal = field(
        default=None,
        metadata={
            "name": "IntrstRate",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    sprd: None | Decimal = field(
        default=None,
        metadata={
            "name": "Sprd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "total_digits": 11,
            "fraction_digits": 10,
        },
    )
    acrd_intrst_amt: AmountAndDirection20 = field(
        metadata={
            "name": "AcrdIntrstAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    aggtd_intrst_amt: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "AggtdIntrstAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class Obligation4:
    pty_a: PartyIdentification100Choice = field(
        metadata={
            "name": "PtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    svcg_pty_a: None | PartyIdentification100Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    pty_b: PartyIdentification100Choice = field(
        metadata={
            "name": "PtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    svcg_pty_b: None | PartyIdentification100Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    coll_acct_id: None | CollateralAccount2 = field(
        default=None,
        metadata={
            "name": "CollAcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    xpsr_tp: None | ExposureType5Code = field(
        default=None,
        metadata={
            "name": "XpsrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    valtn_dt: DateAndDateTimeChoice = field(
        metadata={
            "name": "ValtnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class InterestStatement4:
    intrst_prd: DatePeriodDetails = field(
        metadata={
            "name": "IntrstPrd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    ttl_intrst_amt_due_to_a: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlIntrstAmtDueToA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    ttl_intrst_amt_due_to_b: None | ActiveCurrencyAndAmount = field(
        default=None,
        metadata={
            "name": "TtlIntrstAmtDueToB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    val_dt: XmlDate = field(
        metadata={
            "name": "ValDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    intrst_pmt_req_id: None | str = field(
        default=None,
        metadata={
            "name": "IntrstPmtReqId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "min_length": 1,
            "max_length": 35,
        },
    )
    intrst_clctn: list[InterestCalculation4] = field(
        default_factory=list,
        metadata={
            "name": "IntrstClctn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class InterestPaymentStatementV04:
    tx_id: str = field(
        metadata={
            "name": "TxId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    agrmt: None | Agreement4 = field(
        default=None,
        metadata={
            "name": "Agrmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    oblgtn: Obligation4 = field(
        metadata={
            "name": "Oblgtn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    stmt_params: Statement32 = field(
        metadata={
            "name": "StmtParams",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    pgntn: None | Pagination = field(
        default=None,
        metadata={
            "name": "Pgntn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )
    intrst_stmt: InterestStatement4 = field(
        metadata={
            "name": "IntrstStmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
            "required": True,
        }
    )
    splmtry_data: list[SupplementaryData1] = field(
        default_factory=list,
        metadata={
            "name": "SplmtryData",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04",
        },
    )


@dataclass(kw_only=True)
class Document:
    class Meta:
        namespace = "urn:iso:std:iso:20022:tech:xsd:colr.015.001.04"

    intrst_pmt_stmt: InterestPaymentStatementV04 = field(
        metadata={
            "name": "IntrstPmtStmt",
            "type": "Element",
            "required": True,
        }
    )



from pyiso20022.admi.admi_010_001_02.admi_010_001_02 import (
    Document,
    ReportParameter1,
    RequestDetails4,
    RequestDetails5,
    StaticDataReportV02,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
)

__all__ = [
    "Document",
    "ReportParameter1",
    "RequestDetails4",
    "RequestDetails5",
    "StaticDataReportV02",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
]



from __future__ import annotations

from dataclasses import dataclass, field

from xsdata.models.datatype import XmlDateTime

__NAMESPACE__ = "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01"


@dataclass(kw_only=True)
class MessageReference:
    ref: str = field(
        metadata={
            "name": "Ref",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )


@dataclass(kw_only=True)
class RejectionReason2:
    rjctg_pty_rsn: str = field(
        metadata={
            "name": "RjctgPtyRsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    rjctn_dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "RjctnDtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
        },
    )
    err_lctn: None | str = field(
        default=None,
        metadata={
            "name": "ErrLctn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "min_length": 1,
            "max_length": 350,
        },
    )
    rsn_desc: None | str = field(
        default=None,
        metadata={
            "name": "RsnDesc",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "min_length": 1,
            "max_length": 350,
        },
    )
    addtl_data: None | str = field(
        default=None,
        metadata={
            "name": "AddtlData",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "min_length": 1,
            "max_length": 20000,
        },
    )


@dataclass(kw_only=True)
class Admi00200101:
    class Meta:
        name = "admi.002.001.01"

    rltd_ref: MessageReference = field(
        metadata={
            "name": "RltdRef",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "required": True,
        }
    )
    rsn: RejectionReason2 = field(
        metadata={
            "name": "Rsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class Document:
    class Meta:
        namespace = "urn:iso:std:iso:20022:tech:xsd:admi.002.001.01"

    admi_002_001_01: Admi00200101 = field(
        metadata={
            "name": "admi.002.001.01",
            "type": "Element",
            "required": True,
        }
    )



from __future__ import annotations

from dataclasses import dataclass, field
from decimal import Decimal
from enum import Enum

from xsdata.models.datatype import XmlDate, XmlDateTime

__NAMESPACE__ = "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05"


@dataclass(kw_only=True)
class ActiveOrHistoricCurrencyAndAmount:
    value: Decimal = field(
        metadata={
            "required": True,
            "min_inclusive": Decimal("0"),
            "total_digits": 18,
            "fraction_digits": 5,
        }
    )
    ccy: str = field(
        metadata={
            "name": "Ccy",
            "type": "Attribute",
            "required": True,
            "pattern": r"[A-Z]{3,3}",
        }
    )


class CollateralAccountType1Code(Enum):
    HOUS = "HOUS"
    CLIE = "CLIE"
    LIPR = "LIPR"
    MGIN = "MGIN"
    DFLT = "DFLT"


class CollateralManagementCancellationReason1Code(Enum):
    PRER = "PRER"
    PNSU = "PNSU"


class CollateralRole1Code(Enum):
    GIVE = "GIVE"
    TAKE = "TAKE"


class CollateralTransactionType1Code(Enum):
    AADJ = "AADJ"
    CDTA = "CDTA"
    CADJ = "CADJ"
    DADJ = "DADJ"
    DBVT = "DBVT"
    INIT = "INIT"
    MADJ = "MADJ"
    PADJ = "PADJ"
    RATA = "RATA"
    TERM = "TERM"


class CreditDebitCode(Enum):
    CRDT = "CRDT"
    DBIT = "DBIT"


@dataclass(kw_only=True)
class DateAndDateTime2Choice:
    dt: None | XmlDate = field(
        default=None,
        metadata={
            "name": "Dt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    dt_tm: None | XmlDateTime = field(
        default=None,
        metadata={
            "name": "DtTm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


class DateType2Code(Enum):
    OPEN = "OPEN"


class ExposureType5Code(Enum):
    BFWD = "BFWD"
    PAYM = "PAYM"
    CCPC = "CCPC"
    COMM = "COMM"
    CRDS = "CRDS"
    CRTL = "CRTL"
    CRSP = "CRSP"
    CCIR = "CCIR"
    CRPR = "CRPR"
    EQUI = "EQUI"
    EQPT = "EQPT"
    EQUS = "EQUS"
    EXTD = "EXTD"
    EXPT = "EXPT"
    FIXI = "FIXI"
    FORX = "FORX"
    FORW = "FORW"
    FUTR = "FUTR"
    OPTN = "OPTN"
    LIQU = "LIQU"
    OTCD = "OTCD"
    REPO = "REPO"
    RVPO = "RVPO"
    SLOA = "SLOA"
    SBSC = "SBSC"
    SCRP = "SCRP"
    SLEB = "SLEB"
    SHSL = "SHSL"
    SCIR = "SCIR"
    SCIE = "SCIE"
    SWPT = "SWPT"
    TBAS = "TBAS"
    TRBD = "TRBD"
    TRCP = "TRCP"


@dataclass(kw_only=True)
class GenericIdentification30:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "pattern": r"[a-zA-Z0-9]{4}",
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class GenericIdentification36:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    issr: str = field(
        metadata={
            "name": "Issr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    schme_nm: None | str = field(
        default=None,
        metadata={
            "name": "SchmeNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class PostalAddress2:
    strt_nm: None | str = field(
        default=None,
        metadata={
            "name": "StrtNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )
    pst_cd_id: str = field(
        metadata={
            "name": "PstCdId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 16,
        }
    )
    twn_nm: str = field(
        metadata={
            "name": "TwnNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    ctry_sub_dvsn: None | str = field(
        default=None,
        metadata={
            "name": "CtrySubDvsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    ctry: str = field(
        metadata={
            "name": "Ctry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "pattern": r"[A-Z]{2,2}",
        }
    )


@dataclass(kw_only=True)
class Reference3Choice:
    clnt_coll_instr_id: None | str = field(
        default=None,
        metadata={
            "name": "ClntCollInstrId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    clnt_coll_tx_id: None | str = field(
        default=None,
        metadata={
            "name": "ClntCollTxId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_prpsl_id: None | str = field(
        default=None,
        metadata={
            "name": "CollPrpslId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_prpsl_rspn_id: None | str = field(
        default=None,
        metadata={
            "name": "CollPrpslRspnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_sbstitn_conf_id: None | str = field(
        default=None,
        metadata={
            "name": "CollSbstitnConfId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_sbstitn_req_id: None | str = field(
        default=None,
        metadata={
            "name": "CollSbstitnReqId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    coll_sbstitn_rspn_id: None | str = field(
        default=None,
        metadata={
            "name": "CollSbstitnRspnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    cmon_tx_id: None | str = field(
        default=None,
        metadata={
            "name": "CmonTxId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 52,
        },
    )
    dspt_ntfctn_id: None | str = field(
        default=None,
        metadata={
            "name": "DsptNtfctnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    intrst_pmt_req_id: None | str = field(
        default=None,
        metadata={
            "name": "IntrstPmtReqId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    intrst_pmt_rspn_id: None | str = field(
        default=None,
        metadata={
            "name": "IntrstPmtRspnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    intrst_pmt_stmt_id: None | str = field(
        default=None,
        metadata={
            "name": "IntrstPmtStmtId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    mrgn_call_req_id: None | str = field(
        default=None,
        metadata={
            "name": "MrgnCallReqId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    mrgn_call_rspn_id: None | str = field(
        default=None,
        metadata={
            "name": "MrgnCallRspnId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    trpty_agt_svc_prvdr_coll_instr_id: None | str = field(
        default=None,
        metadata={
            "name": "TrptyAgtSvcPrvdrCollInstrId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    trpty_agt_svc_prvdr_coll_tx_id: None | str = field(
        default=None,
        metadata={
            "name": "TrptyAgtSvcPrvdrCollTxId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )


@dataclass(kw_only=True)
class SupplementaryDataEnvelope1:
    any_element: None | object = field(
        default=None,
        metadata={
            "type": "Wildcard",
            "namespace": "##any",
        },
    )


@dataclass(kw_only=True)
class AmountAndDirection20:
    amt: ActiveOrHistoricCurrencyAndAmount = field(
        metadata={
            "name": "Amt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    cdt_dbt_ind: None | CreditDebitCode = field(
        default=None,
        metadata={
            "name": "CdtDbtInd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralAccountIdentificationType3Choice:
    tp: None | CollateralAccountType1Code = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    prtry: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralCancellationType1Choice:
    cd: None | CollateralManagementCancellationReason1Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralTransactionType1Choice:
    cd: None | CollateralTransactionType1Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class Date3Choice:
    cd: None | DateType2Code = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    prtry: None | GenericIdentification30 = field(
        default=None,
        metadata={
            "name": "Prtry",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class NameAndAddress6:
    nm: str = field(
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 70,
        }
    )
    adr: PostalAddress2 = field(
        metadata={
            "name": "Adr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class SupplementaryData1:
    plc_and_nm: None | str = field(
        default=None,
        metadata={
            "name": "PlcAndNm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 350,
        },
    )
    envlp: SupplementaryDataEnvelope1 = field(
        metadata={
            "name": "Envlp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class ClosingDate4Choice:
    dt: None | DateAndDateTime2Choice = field(
        default=None,
        metadata={
            "name": "Dt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    cd: None | Date3Choice = field(
        default=None,
        metadata={
            "name": "Cd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralAccount3:
    id: str = field(
        metadata={
            "name": "Id",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
            "min_length": 1,
            "max_length": 35,
        }
    )
    tp: None | CollateralAccountIdentificationType3Choice = field(
        default=None,
        metadata={
            "name": "Tp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    nm: None | str = field(
        default=None,
        metadata={
            "name": "Nm",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 70,
        },
    )


@dataclass(kw_only=True)
class CollateralCancellationReason1:
    addtl_inf: None | str = field(
        default=None,
        metadata={
            "name": "AddtlInf",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "min_length": 1,
            "max_length": 35,
        },
    )
    cxl_rsn_cd: CollateralCancellationType1Choice = field(
        metadata={
            "name": "CxlRsnCd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PartyIdentification178Choice:
    any_bic: None | str = field(
        default=None,
        metadata={
            "name": "AnyBIC",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "pattern": r"[A-Z0-9]{4,4}[A-Z]{2,2}[A-Z0-9]{2,2}([A-Z0-9]{3,3}){0,1}",
        },
    )
    prtry_id: None | GenericIdentification36 = field(
        default=None,
        metadata={
            "name": "PrtryId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    nm_and_adr: None | NameAndAddress6 = field(
        default=None,
        metadata={
            "name": "NmAndAdr",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class Obligation7:
    pty_a: PartyIdentification178Choice = field(
        metadata={
            "name": "PtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    svcg_pty_a: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyA",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    pty_b: PartyIdentification178Choice = field(
        metadata={
            "name": "PtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    svcg_pty_b: None | PartyIdentification178Choice = field(
        default=None,
        metadata={
            "name": "SvcgPtyB",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    coll_acct_id: None | CollateralAccount3 = field(
        default=None,
        metadata={
            "name": "CollAcctId",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    xpsr_tp: None | ExposureType5Code = field(
        default=None,
        metadata={
            "name": "XpsrTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    coll_tx_tp: None | CollateralTransactionType1Choice = field(
        default=None,
        metadata={
            "name": "CollTxTp",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    coll_sd: None | CollateralRole1Code = field(
        default=None,
        metadata={
            "name": "CollSd",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    xpsr_amt: None | AmountAndDirection20 = field(
        default=None,
        metadata={
            "name": "XpsrAmt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    valtn_dt: None | DateAndDateTime2Choice = field(
        default=None,
        metadata={
            "name": "ValtnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    clsg_dt: None | ClosingDate4Choice = field(
        default=None,
        metadata={
            "name": "ClsgDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )
    reqd_exctn_dt: None | DateAndDateTime2Choice = field(
        default=None,
        metadata={
            "name": "ReqdExctnDt",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class CollateralManagementCancellationRequestV05:
    ref: Reference3Choice = field(
        metadata={
            "name": "Ref",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    oblgtn: Obligation7 = field(
        metadata={
            "name": "Oblgtn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    cxl_rsn: CollateralCancellationReason1 = field(
        metadata={
            "name": "CxlRsn",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
            "required": True,
        }
    )
    splmtry_data: list[SupplementaryData1] = field(
        default_factory=list,
        metadata={
            "name": "SplmtryData",
            "type": "Element",
            "namespace": "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05",
        },
    )


@dataclass(kw_only=True)
class Document:
    class Meta:
        namespace = "urn:iso:std:iso:20022:tech:xsd:colr.005.001.05"

    coll_mgmt_cxl_req: CollateralManagementCancellationRequestV05 = field(
        metadata={
            "name": "CollMgmtCxlReq",
            "type": "Element",
            "required": True,
        }
    )



"""ISO20022 Dataclass Validation System.
This module provides validation functionality for ISO20022 dataclasses based on
their metadata constraints without requiring XSD files. It extracts validation
rules from field metadata and applies them to validate field values.
"""
import inspect
import re
from dataclasses import fields, is_dataclass
from decimal import Decimal
from enum import Enum
from typing import Any, Dict, List, Optional, Type, Union

# XmlDate, XmlDateTime, XmlTime are available for future use


class ValidationError(Exception):
    """Raised when validation fails."""
    def __init__(self, field_name: str, value: Any, constraint: str,
                 message: str):
        """Initialize ValidationError.
        Args:
            field_name: Name of the field that failed validation
            value: The value that failed validation
            constraint: The constraint that was violated
            message: Human-readable error message
        """
        self.field_name = field_name
        self.value = value
        self.constraint = constraint
        self.message = message
        super().__init__(f"Field '{field_name}': {message}")


class ValidationResult:
    """Contains the results of a validation operation."""
    def __init__(self):
        """Initialize ValidationResult."""
        self.errors: List[ValidationError] = []
        self.warnings: List[str] = []
        self.is_valid = True

    def add_error(self, field_name: str, value: Any, constraint: str,
                  message: str):
        """Add a validation error.
        Args:
            field_name: Name of the field that failed validation
            value: The value that failed validation
            constraint: The constraint that was violated
            message: Human-readable error message
        """
        error = ValidationError(field_name, value, constraint, message)
        self.errors.append(error)
        self.is_valid = False

    def add_warning(self, message: str):
        """Add a validation warning.
        Args:
            message: Warning message
        """
        self.warnings.append(message)

    def to_yaml(self) -> str:
        """Return YAML representation of validation result."""
        import yaml

        result_dict = {
            'validation_status': 'PASSED' if self.is_valid else 'FAILED',
            'error_count': len(self.errors),
            'warning_count': len(self.warnings)
        }

        if self.errors:
            result_dict['errors'] = []
            for error in self.errors:
                error_dict = {
                    'field': error.field_name,
                    'constraint': error.constraint,
                    'value': (str(error.value) if error.value is not None
                              else None),
                    'message': error.message
                }
                result_dict['errors'].append(error_dict)

        if self.warnings:
            result_dict['warnings'] = self.warnings

        return yaml.dump(result_dict, default_flow_style=False,
                         sort_keys=False)

    def to_summary_yaml(self) -> str:
        """Return concise YAML summary of validation result."""
        import yaml

        summary = {
            'status': 'PASSED' if self.is_valid else 'FAILED',
            'errors': len(self.errors),
            'warnings': len(self.warnings)
        }

        if not self.is_valid and self.errors:
            # Group errors by constraint type
            error_types = {}
            for error in self.errors:
                constraint = error.constraint
                if constraint not in error_types:
                    error_types[constraint] = 0
                error_types[constraint] += 1
            summary['error_breakdown'] = error_types

        return yaml.dump(summary, default_flow_style=False, sort_keys=False)

    def __str__(self):
        """Return string representation of validation result."""
        if self.is_valid:
            return "Validation passed"
        result = f"Validation failed with {len(self.errors)} error(s):\n"
        for error in self.errors:
            result += f"  - {error}\n"
        if self.warnings:
            result += f"\nWarnings ({len(self.warnings)}):\n"
            for warning in self.warnings:
                result += f"  - {warning}\n"
        return result


class ISO20022Validator:
    """Validates ISO20022 dataclass instances based on metadata constraints."""
    def __init__(self):
        """Initialize ISO20022Validator."""
        self.type_validators = {
            'pattern': self._validate_pattern,
            'min_length': self._validate_min_length,
            'max_length': self._validate_max_length,
            'min_inclusive': self._validate_min_inclusive,
            'max_inclusive': self._validate_max_inclusive,
            'min_exclusive': self._validate_min_exclusive,
            'max_exclusive': self._validate_max_exclusive,
            'total_digits': self._validate_total_digits,
            'fraction_digits': self._validate_fraction_digits,
            'required': self._validate_required,
            'min_occurs': self._validate_min_occurs,
            'max_occurs': self._validate_max_occurs,
        }
        self._validation_path = []  # Track validation path for nested objects

    def validate_message(self, instance: Any,
                         strict: bool = True) -> ValidationResult:
        """Validate a complete ISO20022 message recursively.
        This method validates an entire message structure, including all nested
        dataclass instances, lists, and their constraints.
        Args:
            instance: The message instance to validate (typically Document or
                     root)
            strict: If True, stops on first error. If False, collects all
                   errors.
        Returns:
            ValidationResult containing validation results for entire message
        """
        self._validation_path = []
        return self._validate_recursive(instance, strict)

    def _validate_recursive(self, instance: Any,
                            strict: bool) -> ValidationResult:
        """Recursively validate an instance and all its nested components.
        Args:
            instance: The instance to validate
            strict: Whether to stop on first error
        Returns:
            ValidationResult containing all validation errors found
        """
        result = ValidationResult()
        if instance is None:
            return result
        # Handle lists
        if isinstance(instance, list):
            for i, item in enumerate(instance):
                self._validation_path.append(f"[{i}]")
                item_result = self._validate_recursive(item, strict)
                self._merge_results(result, item_result)
                self._validation_path.pop()
                if strict and not result.is_valid:
                    break
            return result
        # Handle dataclass instances
        if is_dataclass(instance):
            # First validate the current instance
            current_result = self.validate(instance, strict=False)
            self._merge_results(result, current_result)
            if strict and not result.is_valid:
                return result
            # Then recursively validate nested fields
            dataclass_fields = fields(instance)
            for field in dataclass_fields:
                field_value = getattr(instance, field.name)
                if field_value is not None:
                    self._validation_path.append(field.name)
                    # Recursively validate nested objects
                    nested_result = self._validate_recursive(field_value,
                                                             strict)
                    self._merge_results(result, nested_result)
                    self._validation_path.pop()
                    if strict and not result.is_valid:
                        break
        return result

    def _merge_results(self, target: ValidationResult,
                       source: ValidationResult):
        """Merge validation results, updating field names with path context.
        Args:
            target: The target ValidationResult to merge into
            source: The source ValidationResult to merge from
        """
        path_prefix = ".".join(self._validation_path)
        for error in source.errors:
            # Update field name with full path, avoiding duplicates
            if path_prefix and not error.field_name.startswith(path_prefix):
                full_field_name = f"{path_prefix}.{error.field_name}"
            else:
                full_field_name = error.field_name
            target.add_error(full_field_name, error.value,
                             error.constraint, error.message)
        for warning in source.warnings:
            # Update warning with path context if needed
            full_warning = (f"{path_prefix}: {warning}"
                            if path_prefix else warning)
            target.add_warning(full_warning)

    def validate(self, instance: Any, strict: bool = True) -> ValidationResult:
        """Validate a dataclass instance against its metadata constraints.
        Args:
            instance: The dataclass instance to validate
            strict: If True, raises ValidationError on first failure.
                   If False, collects all errors.
        Returns:
            ValidationResult containing validation results
        Raises:
            ValueError: If instance is not a dataclass
        """
        if not is_dataclass(instance):
            raise ValueError("Instance must be a dataclass")
        result = ValidationResult()
        # Get all fields for this dataclass
        dataclass_fields = fields(instance)
        for field in dataclass_fields:
            field_value = getattr(instance, field.name)
            self._validate_field(field, field_value, result, strict)
            if strict and not result.is_valid:
                break
        return result

    def validate_field_value(self, dataclass_type: Type, field_name: str,
                             value: Any) -> ValidationResult:
        """Validate a single field value against its constraints.
        Args:
            dataclass_type: The dataclass type containing the field
            field_name: Name of the field to validate
            value: Value to validate
        Returns:
            ValidationResult containing validation results
        Raises:
            ValueError: If dataclass_type is not a dataclass or field not found
        """
        if not is_dataclass(dataclass_type):
            raise ValueError("dataclass_type must be a dataclass")
        # Find the field
        dataclass_fields = fields(dataclass_type)
        target_field = None
        for field in dataclass_fields:
            if field.name == field_name:
                target_field = field
                break
        if target_field is None:
            raise ValueError(
                f"Field '{field_name}' not found in {dataclass_type.__name__}"
            )
        result = ValidationResult()
        self._validate_field(target_field, value, result, strict=False)
        return result

    def _validate_field(self, field, value: Any, result: ValidationResult,
                        strict: bool):
        """Validate a single field against its metadata constraints.
        Args:
            field: The dataclass field to validate
            value: The value to validate
            result: ValidationResult to add errors to
            strict: Whether to stop on first error
        """
        metadata = field.metadata
        field_name = field.name
        # Skip validation if value is None and field is optional
        if value is None:
            if metadata.get('required', False):
                result.add_error(
                    field_name, value, 'required',
                    'Field is required but value is None'
                )
            return

        # Special handling for list fields
        if isinstance(value, list):
            # First validate list-specific constraints (min_occurs, max_occurs)
            self._validate_list_constraints(field, value, result)

            # For lists, apply string constraints to each item,
            # not the list itself
            string_constraints = ['min_length', 'max_length', 'pattern']
            list_item_constraints = {k: v for k, v in metadata.items()
                                     if k in string_constraints}

            # Validate each item in the list
            if list_item_constraints:
                for i, item in enumerate(value):
                    for constraint_name, constraint_value in (
                            list_item_constraints.items()):
                        if constraint_name in self.type_validators:
                            try:
                                self.type_validators[constraint_name](
                                    f"{field_name}[{i}]", item,
                                    constraint_value, result
                                )
                            except Exception as e:
                                result.add_error(
                                    f"{field_name}[{i}]", item,
                                    constraint_name,
                                    f"Validation error: {str(e)}"
                                )
                            if strict and not result.is_valid:
                                break
                    if strict and not result.is_valid:
                        break

            # Skip the normal constraint validation for lists
            non_list_constraints = {
                k: v for k, v in metadata.items()
                if (k in self.type_validators and
                    k not in string_constraints and
                    k not in ['min_occurs', 'max_occurs'])
            }
        else:
            # For non-list fields, validate all constraints normally
            non_list_constraints = {k: v for k, v in metadata.items()
                                    if k in self.type_validators}

        # Validate non-list constraints
        for constraint_name, constraint_value in non_list_constraints.items():
            try:
                self.type_validators[constraint_name](
                    field_name, value, constraint_value, result
                )
            except Exception as e:
                result.add_error(
                    field_name, value, constraint_name,
                    f"Validation error: {str(e)}"
                )
            if strict and not result.is_valid:
                break

        # Validate enum values
        enum_type = self._get_enum_type(field.type)
        if enum_type and value is not None:
            self._validate_enum(field_name, value, enum_type, result)

    def _get_enum_type(self, field_type) -> Optional[Type[Enum]]:
        """Extract enum type from field type annotation.
        Args:
            field_type: The field type annotation
        Returns:
            Enum type if found, None otherwise
        """
        if (hasattr(field_type, '__origin__') and
                field_type.__origin__ is Union):
            # Handle Optional[EnumType] case
            args = field_type.__args__
            for arg in args:
                if (arg is not type(None) and inspect.isclass(arg) and
                        issubclass(arg, Enum)):
                    return arg
        elif inspect.isclass(field_type) and issubclass(field_type, Enum):
            return field_type
        return None

    def _validate_pattern(self, field_name: str, value: Any, pattern: str,
                          result: ValidationResult):
        """Validate regex pattern constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            pattern: Regex pattern to match
            result: ValidationResult to add errors to
        """
        if not isinstance(value, str):
            return  # Pattern only applies to strings
        try:
            if not re.match(pattern, value):
                result.add_error(
                    field_name, value, 'pattern',
                    f"Value '{value}' does not match required pattern "
                    f"'{pattern}'"
                )
        except re.error as e:
            result.add_error(
                field_name, value, 'pattern',
                f"Invalid regex pattern '{pattern}': {str(e)}"
            )

    def _validate_min_length(self, field_name: str, value: Any,
                             min_length: int, result: ValidationResult):
        """Validate minimum length constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            min_length: Minimum required length
            result: ValidationResult to add errors to
        """
        if hasattr(value, '__len__'):
            if len(value) < min_length:
                result.add_error(
                    field_name, value, 'min_length',
                    f"Length {len(value)} is less than minimum required "
                    f"length {min_length}"
                )

    def _validate_max_length(self, field_name: str, value: Any,
                             max_length: int, result: ValidationResult):
        """Validate maximum length constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            max_length: Maximum allowed length
            result: ValidationResult to add errors to
        """
        if hasattr(value, '__len__'):
            if len(value) > max_length:
                result.add_error(
                    field_name, value, 'max_length',
                    f"Length {len(value)} exceeds maximum allowed "
                    f"length {max_length}"
                )

    def _validate_min_inclusive(self, field_name: str, value: Any,
                                min_value: Union[int, float, Decimal],
                                result: ValidationResult):
        """Validate minimum inclusive constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            min_value: Minimum allowed value (inclusive)
            result: ValidationResult to add errors to
        """
        if isinstance(value, (int, float, Decimal)):
            if value < min_value:
                result.add_error(
                    field_name, value, 'min_inclusive',
                    f"Value {value} is less than minimum allowed "
                    f"value {min_value}"
                )

    def _validate_max_inclusive(self, field_name: str, value: Any,
                                max_value: Union[int, float, Decimal],
                                result: ValidationResult):
        """Validate maximum inclusive constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            max_value: Maximum allowed value (inclusive)
            result: ValidationResult to add errors to
        """
        if isinstance(value, (int, float, Decimal)):
            if value > max_value:
                result.add_error(
                    field_name, value, 'max_inclusive',
                    f"Value {value} exceeds maximum allowed "
                    f"value {max_value}"
                )

    def _validate_min_exclusive(self, field_name: str, value: Any,
                                min_value: Union[int, float, Decimal],
                                result: ValidationResult):
        """Validate minimum exclusive constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            min_value: Minimum value (exclusive)
            result: ValidationResult to add errors to
        """
        if isinstance(value, (int, float, Decimal)):
            if value <= min_value:
                result.add_error(
                    field_name, value, 'min_exclusive',
                    f"Value {value} must be greater than {min_value}"
                )

    def _validate_max_exclusive(self, field_name: str, value: Any,
                                max_value: Union[int, float, Decimal],
                                result: ValidationResult):
        """Validate maximum exclusive constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            max_value: Maximum value (exclusive)
            result: ValidationResult to add errors to
        """
        if isinstance(value, (int, float, Decimal)):
            if value >= max_value:
                result.add_error(
                    field_name, value, 'max_exclusive',
                    f"Value {value} must be less than {max_value}"
                )

    def _validate_total_digits(self, field_name: str, value: Any,
                               total_digits: int, result: ValidationResult):
        """Validate total digits constraint for Decimal values.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            total_digits: Maximum total digits allowed
            result: ValidationResult to add errors to
        """
        if isinstance(value, Decimal):
            # Convert to string and count digits (excluding decimal point and
            # sign)
            value_str = str(abs(value)).replace('.', '')
            if len(value_str) > total_digits:
                result.add_error(
                    field_name, value, 'total_digits',
                    f"Total digits {len(value_str)} exceeds maximum "
                    f"allowed {total_digits}"
                )

    def _validate_fraction_digits(self, field_name: str, value: Any,
                                  fraction_digits: int,
                                  result: ValidationResult):
        """Validate fraction digits constraint for Decimal values.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            fraction_digits: Maximum fraction digits allowed
            result: ValidationResult to add errors to
        """
        if isinstance(value, Decimal):
            # Get the number of decimal places
            exponent = value.as_tuple().exponent
            decimal_places = abs(exponent) if isinstance(exponent, int) else 0
            if decimal_places > fraction_digits:
                result.add_error(
                    field_name, value, 'fraction_digits',
                    f"Fraction digits {decimal_places} exceeds maximum "
                    f"allowed {fraction_digits}"
                )

    def _validate_required(self, field_name: str, value: Any, required: bool,
                           result: ValidationResult):
        """Validate required constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            required: Whether field is required
            result: ValidationResult to add errors to
        """
        if required and value is None:
            result.add_error(
                field_name, value, 'required',
                "Field is required but value is None"
            )

    def _validate_min_occurs(self, field_name: str, value: Any,
                             min_occurs: int, result: ValidationResult):
        """Validate minimum occurrences constraint for lists.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            min_occurs: Minimum number of items required
            result: ValidationResult to add errors to
        """
        if isinstance(value, list):
            if len(value) < min_occurs:
                result.add_error(
                    field_name, value, 'min_occurs',
                    f"List has {len(value)} items but minimum required "
                    f"is {min_occurs}"
                )

    def _validate_max_occurs(self, field_name: str, value: Any,
                             max_occurs: int, result: ValidationResult):
        """Validate maximum occurrences constraint for lists.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            max_occurs: Maximum number of items allowed
            result: ValidationResult to add errors to
        """
        if isinstance(value, list):
            if len(value) > max_occurs:
                result.add_error(
                    field_name, value, 'max_occurs',
                    f"List has {len(value)} items but maximum allowed "
                    f"is {max_occurs}"
                )

    def _validate_enum(self, field_name: str, value: Any,
                       enum_type: Type[Enum], result: ValidationResult):
        """Validate enum value constraint.
        Args:
            field_name: Name of the field being validated
            value: Value to validate
            enum_type: The enum type to validate against
            result: ValidationResult to add errors to
        """
        try:
            if isinstance(value, enum_type):
                return  # Already a valid enum instance
            # Try to convert string to enum
            if isinstance(value, str):
                enum_type(value)
            else:
                result.add_error(
                    field_name, value, 'enum',
                    f"Value '{value}' is not a valid {enum_type.__name__} "
                    f"enum value"
                )
        except ValueError:
            valid_values = [e.value for e in enum_type]
            result.add_error(
                field_name, value, 'enum',
                f"Value '{value}' is not a valid {enum_type.__name__} "
                f"enum value. Valid values: {valid_values}"
            )

    def _validate_list_constraints(self, field, value: list,
                                   result: ValidationResult):
        """Validate constraints specific to list fields.
        Args:
            field: The dataclass field being validated
            value: List value to validate
            result: ValidationResult to add errors to
        """
        metadata = field.metadata
        field_name = field.name
        # Validate min_occurs and max_occurs
        if 'min_occurs' in metadata:
            self._validate_min_occurs(
                field_name, value, metadata['min_occurs'], result
            )
        if 'max_occurs' in metadata:
            self._validate_max_occurs(
                field_name, value, metadata['max_occurs'], result
            )

    def get_field_constraints(self, dataclass_type: Type) -> Dict[str, Dict[str, Any]]:  # noqa: E501
        """Extract all validation constraints for fields in a dataclass.
        Args:
            dataclass_type: The dataclass type to analyze
        Returns:
            Dictionary mapping field names to their constraints
        Raises:
            ValueError: If dataclass_type is not a dataclass
        """
        if not is_dataclass(dataclass_type):
            raise ValueError("dataclass_type must be a dataclass")
        constraints = {}
        dataclass_fields = fields(dataclass_type)
        for field in dataclass_fields:
            field_constraints = {}
            # Extract validation constraints from metadata
            for key, value in field.metadata.items():
                if (key in self.type_validators or
                        key in ['type', 'namespace', 'name']):
                    field_constraints[key] = value
            # Add type information
            field_constraints['field_type'] = field.type
            field_constraints['default'] = (
                field.default if field.default is not field.default_factory
                else None
            )
            constraints[field.name] = field_constraints
        return constraints


def validate_instance(instance: Any, strict: bool = True) -> ValidationResult:
    """Convenience function to validate a dataclass instance.
    Args:
        instance: The dataclass instance to validate
        strict: If True, stops on first error. If False, collects all errors.
    Returns:
        ValidationResult containing validation results
    """
    validator = ISO20022Validator()
    return validator.validate(instance, strict)


def validate_message(instance: Any, strict: bool = True) -> ValidationResult:
    """Convenience function to validate a complete ISO20022 message.
    Args:
        instance: The message instance to validate (typically Document or root)
        strict: If True, stops on first error. If False, collects all errors.
    Returns:
        ValidationResult containing validation results for entire message
    """
    validator = ISO20022Validator()
    return validator.validate_message(instance, strict)


def validate_field(dataclass_type: Type, field_name: str,
                   value: Any) -> ValidationResult:
    """Convenience function to validate a single field value.
    Args:
        dataclass_type: The dataclass type containing the field
        field_name: Name of the field to validate
        value: Value to validate
    Returns:
        ValidationResult containing validation results
    """
    validator = ISO20022Validator()
    return validator.validate_field_value(dataclass_type, field_name, value)


def get_constraints(dataclass_type: Type) -> Dict[str, Dict[str, Any]]:
    """Convenience function to get all constraints for a dataclass.
    Args:
        dataclass_type: The dataclass type to analyze
    Returns:
        Dictionary mapping field names to their constraints
    """
    validator = ISO20022Validator()
    return validator.get_field_constraints(dataclass_type)



from pyiso20022.colr.colr_024_001_01.colr_024_001_01 import (
    ActiveCurrencyAndAmount,
    ActiveOrHistoricCurrencyAndAmount,
    AddressType2Code,
    AlternatePartyIdentification7,
    AmountAndDirection49,
    BlockChainAddressWallet3,
    CollateralParties8,
    CollateralRole1Code,
    CollateralTransactionType1Choice,
    CollateralTransactionType1Code,
    CreditDebitCode,
    DateAndDateTime2Choice,
    Document,
    ExposureType14Code,
    ExposureType23Choice,
    ForeignExchangeTerms23,
    GenericIdentification1,
    GenericIdentification30,
    GenericIdentification36,
    IdentificationType42Choice,
    NameAndAddress5,
    PartyIdentification120Choice,
    PartyIdentification136,
    PartyIdentificationAndAccount193,
    PartyIdentificationAndAccount202,
    PartyIdentificationAndAccount203,
    PostalAddress1,
    SecuritiesAccount19,
    SupplementaryData1,
    SupplementaryDataEnvelope1,
    TradingCapacity7Code,
    TradingPartyCapacity5Choice,
    TransactionIdentifications44,
    TripartyCollateralAllegementNotificationCancellationAdviceV01,
    TypeOfIdentification1Code,
)

__all__ = [
    "ActiveCurrencyAndAmount",
    "ActiveOrHistoricCurrencyAndAmount",
    "AddressType2Code",
    "AlternatePartyIdentification7",
    "AmountAndDirection49",
    "BlockChainAddressWallet3",
    "CollateralParties8",
    "CollateralRole1Code",
    "CollateralTransactionType1Choice",
    "CollateralTransactionType1Code",
    "CreditDebitCode",
    "DateAndDateTime2Choice",
    "Document",
    "ExposureType14Code",
    "ExposureType23Choice",
    "ForeignExchangeTerms23",
    "GenericIdentification1",
    "GenericIdentification30",
    "GenericIdentification36",
    "IdentificationType42Choice",
    "NameAndAddress5",
    "PartyIdentification120Choice",
    "PartyIdentification136",
    "PartyIdentificationAndAccount193",
    "PartyIdentificationAndAccount202",
    "PartyIdentificationAndAccount203",
    "PostalAddress1",
    "SecuritiesAccount19",
    "SupplementaryData1",
    "SupplementaryDataEnvelope1",
    "TradingCapacity7Code",
    "TradingPartyCapacity5Choice",
    "TransactionIdentifications44",
    "TripartyCollateralAllegementNotificationCancellationAdviceV01",
    "TypeOfIdentification1Code",
]
